const http = require('http');
const url = require('url');
const path = require('path');
const fs = require('fs');

const mimeTypes = {
  "html": "text/html",
  "css": "text/css",
  "js": "text/javascript",
  "jpeg": "image/jpeg",
  "jpg": "image/jpeg",
  "png": "image/png",
  "gif": "image/gif",
  "webp": "image/webp",
  "svg": "image/svg+xml",
  "icon": "image/x-icon",
  "webm": "video/webm",
  "ogg": "video/ogg",
  "mp4": "video/mp4",
  "mp3": "audio/mpeg",
  "ttf": "font/ttf",
  "otf": "font/otf",
  "woff": "font/woff",
  "woff2": "font/woff2",
  "pdf": "application/pdf",
  "docx": "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
};

const hostname = 'localhost';
const port = 3069;

const server = http.createServer((req, res) => {
  const parsedUrl = url.parse(req.url);
  const uri = parsedUrl.pathname;
  // searchg c# parser
  if (uri.startsWith('/ddg-api')) {
    const targetPath = req.url.replace(/^\/ddg-api/, '');
   // console.log(`Proxying: http://localhost:50000${targetPath}`);
  // uncomment if you're an asshole and wanna see and log user history. please keep this commented and dont be an asshole!
    const proxyReq = http.request({
      hostname: 'localhost',
      port: 50000,
      path: targetPath,
      method: req.method,
      headers: req.headers
    }, proxyRes => {
      res.writeHead(proxyRes.statusCode, proxyRes.headers);
      proxyRes.pipe(res);
    });

    proxyReq.on('error', () => {
      res.writeHead(502);
      res.end('Bad Gateway');
    });

    req.pipe(proxyReq);
    return;
  }

  const filePath = path.join(process.cwd(), unescape(uri));
  fs.lstat(filePath, (err, stats) => {
    if (err) {
      res.writeHead(404);
      res.end('404 Not Found');
      return;
    }

    if (stats.isFile()) {
      const ext = path.extname(filePath).slice(1);
      res.writeHead(200, { 'Content-Type': mimeTypes[ext] || 'application/octet-stream' });
      fs.createReadStream(filePath).pipe(res);
    } else if (stats.isDirectory()) {
      const indexFile = path.join(filePath, 'index.html');
      fs.exists(indexFile, exists => {
        if (exists) {
          res.writeHead(200, { 'Content-Type': mimeTypes['html'] });
          fs.createReadStream(indexFile).pipe(res);
        } else {
          res.writeHead(404);
          res.end('404 Not Found');
        }
      });
    } else {
      res.writeHead(500);
      res.end('Internal Server Error');
    }
  });
});

server.listen(port, hostname, () => {
  console.log(`searchg running at http://${hostname}:${port}/`);
});
